==> cavity-5.4.tar.gz: cavity-5.4/src/plot.c <==
/* -*- C -*- */
/* $Id: plot.c 131 2006-02-14 08:14:55Z tnemoto $ */

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#include "cavity.h"

static int num_pen;
static double bx0=10000,by0=10000,bx1=-10000,by1=-10000;
static long boundingBoxPointer;

FILE *l_meta;

#define BoundingBoxString "%%%%BoundingBox: %6d %6d %6d %6d\n"

int CavPlotArc(double fx,double fy,double fz,double fr,
	    int ist,int ied,char *name1) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"ca  %8.3f%8.3f%8.3f%5d%5d a %% %8.3f    %-6s\n",
	  fx,fy,fr,ist,ied,fz,name1);
  if (bx1<fx+fr*cost[ist]) {
    bx1=fx+fr*cost[ist];
  }
  if (bx1<fx+fr*cost[ied]) {
    bx1=fx+fr*cost[ied];
  }
  if (ist<=180 && ied>=180 && bx0>fx-fr) {
    bx0=fx-fr;
  } 
  if (fx+fr*cost[ist]<bx0) {
    bx0=fx+fr*cost[ist];
  }
  if (fx+fr*cost[ied]<bx0) {
    bx0=fx+fr*cost[ied];
  }
  if (ist<=90 && ied>=90 && by1<fy+fr) {
    by1=fy+fr;
  } 
  if (by1<fy+fr*sint[ist]) {
    by1=fy+fr*sint[ist];
  }
  if (by1<fy+fr*sint[ied]) {
    by1=fy+fr*sint[ied];
  }
  if (ist<=270 && ied>=270 && by0>fy-fr) {
    by0=fy-fr;
  } 
  if (fy+fr*sint[ist]<by0) {
    by0=fy+fr*sint[ist];
  }
  if (fy+fr*sint[ied]<by0) {
    by0=fy+fr*sint[ied];
  }
  return 1;
}

int CavOpenPlotter(char *fileName) {
#ifdef CREATION_TIME
  time_t tt;
  tt=time(NULL);
#endif  

  l_meta=fopen(fileName,"w+");
  if (l_meta==NULL) error("Can't Open Output File!");

  num_pen=-1;
  fprintf(l_meta,"%s\n","%!PS-Adobe-3.0 EPSF-3.0");
  fprintf(l_meta,"%s\n","%%Creater: PS-META-FILE(Cavity 4.5)");
#ifdef CREATION_TIME
  fprintf(l_meta,"%s %s","%%CreationDate: ",asctime(localtime(&tt)));
#endif
  /*
  c      fprintf(l_meta,"%s\n","%%DocumentMedia: ");
  c      fprintf(l_meta,"%s\n","%%DocumentPrinterRequired: ");
  c      fprintf(l_meta,"%s\n","%%For:");
  c      fprintf(l_meta,"%s\n","%%PageMedia:");
  c      fprintf(l_meta,"%s\n","%%PageRequirements:");
  */
  boundingBoxPointer=ftell(l_meta);
  fprintf(l_meta,BoundingBoxString,0,0,0,0);
  fprintf(l_meta,"%s\n","%%Pages: 1");
  fprintf(l_meta,"%s\n","%%EndComment");
  fprintf(l_meta,"save\n");
  fprintf(l_meta,"/color { false } def\n");
  fprintf(l_meta,"/color2 { false } def\n");
  fprintf(l_meta,"/penwidth {[0.001 0.001 0.005 0.050 0.001]} def\n");
  fprintf(l_meta,"/pengray {[0.000 0.000 0.000 0.000 0.700]} def\n");
  fprintf(l_meta,"/penred {[0.000 0.000 0.000 1.000 0.700]} def\n");
  fprintf(l_meta,"/pengreen {[0.000 0.000 0.000 0.000 0.700]} def\n");
  fprintf(l_meta,"/penblue {[0.000 1.000 0.000 0.000 0.700]} def\n");

  fprintf(l_meta,"/colorblue {[0 1 0 1 0 1 0 1]} def\n");
  fprintf(l_meta,"/colorred {[0 0 1 1 0 0 1 1]} def\n");
  fprintf(l_meta,"/colorgreen {[0 0 0 0 1 1 1 1]} def\n");

  fprintf(l_meta,"/m { moveto } def\n");
  fprintf(l_meta,"/s { stroke } def\n");
  fprintf(l_meta,"/l { /liney exch def /linex exch def\n");
  fprintf(l_meta,"    m linex liney lineto s} def\n");
  fprintf(l_meta,"/a { /arced exch def /arcst exch def\n");
  fprintf(l_meta,"/arcr exch def /arcy exch def\n");
  fprintf(l_meta,"/arcx exch def arcst cos arcr mul arcx add\n");
  fprintf(l_meta,"arcst sin arcr mul arcy add m \n");
  fprintf(l_meta,"arcx arcy arcr arcst arced arc s} def\n");
  fprintf(l_meta,"/C { /col exch def  color2 {\n");
  fprintf(l_meta,"colorred col get\n");
  fprintf(l_meta,"colorgreen col get\n");
  fprintf(l_meta,"colorblue col get\n");
  fprintf(l_meta,"setrgbcolor\n");
  fprintf(l_meta,"} if } def\n");
  fprintf(l_meta,"/p { dup penwidth exch get setlinewidth\n");
  fprintf(l_meta,"     color { /col exch def\n");
  fprintf(l_meta,"        penred col get pengreen col get\n");
  fprintf(l_meta,"        penblue col get setrgbcolor \n");
  fprintf(l_meta,"     } {pengray exch get setgray }\n");
  fprintf(l_meta,"     ifelse  } def\n");
  fprintf(l_meta,"/rm { rmoveto } def\n");
  fprintf(l_meta,"/rl { rlineto } def\n");
  fprintf(l_meta,"/tt {pop} def\n");
  fprintf(l_meta,"/ar { } def\n");
  fprintf(l_meta,"/cm {} def\n");
  fprintf(l_meta,"/cl {} def\n");
  fprintf(l_meta,"/crm {} def\n");
  fprintf(l_meta,"/crl {} def\n");
  fprintf(l_meta,"/cp {} def\n");
  fprintf(l_meta,"/ct {} def\n");
  fprintf(l_meta,"/ca {} def\n");
  fprintf(l_meta,"/cc {} def\n");
  fprintf(l_meta,"/cC {} def\n");
  fprintf(l_meta,"%% 50 50 m 500 0 rl 0 500 rl -500 0 rl\n");
  fprintf(l_meta,"%% closepath clip newpath\n");
  fprintf(l_meta,"%10.4f%10.4f%s\n",300.0,300.0," translate");
  fprintf(l_meta,"%10.4f%10.4f%s\n",50.0,50.0," scale");
  fprintf(l_meta,"%% View Control\n");
  fprintf(l_meta,"%10.4f%10.4f%s\n",scale,scale," scale");
  fprintf(l_meta,"%10.4f%10.4f%s\n",0.0,0.0," translate");
  fprintf(l_meta,"%s\n","%%EndProlog");
  fprintf(l_meta,"%s\n","%# StartData");
  fprintf(l_meta,"%%# ORIGIN=%10.6f%10.6f%10.6f\n",org[0],org[1],org[2]);
  fflush(l_meta);
  return 1;
}

int CavPlotLine(double fx,double fy,double fz,double fx2,double fy2,double fz2,
	    char *name1,char *name2){
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cl  %8.3f%8.3f%8.3f%8.3f l %% %8.3f%8.3f %-6s %-6s\n",
	  fx,fy,fx2,fy2,fz,fz2,name1,name2);
  if (fx<bx0) bx0=fx;
  if (fx2<bx0) bx0=fx2;
  if (fx>bx1) bx1=fx;
  if (fx2>bx1) bx1=fx2;
  if (fy<by0) by0=fy;
  if (fy2<by0) by0=fy2;
  if (fy>by1) by1=fy;
  if (fy2>by1) by1=fy2;
  return 1;
}

int CavPlotPen(int i){
  if (l_meta == NULL) return 0;
  if (i!=num_pen) {
    num_pen=i;
    fprintf(l_meta,"cp  %7d p\n",iPen[i-1]);
  }
  return 1;
}

int CavClosePlotter(void) {
  if (l_meta==NULL) return 0;
  fprintf(l_meta,"%%# EndData\n");
  fprintf(l_meta,"%%%%Trailer\n");
  fprintf(l_meta,"restore\n");
  fprintf(l_meta,"showpage\n");
  fprintf(l_meta,"%%%%EndTrailer\n");
  fprintf(l_meta,"%%%%EOF\n");
  fseek(l_meta,boundingBoxPointer,SEEK_SET);
  fprintf(l_meta,BoundingBoxString,
	  (int)(bx0*50*scale+300),
	  (int)(by0*50*scale+300),
	  (int)(bx1*50*scale+300),
	  (int)(by1*50*scale+300));
  fclose(l_meta);
  l_meta=NULL;
  return 1;
}

int CavPlotTitle(char *title) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"ct (%-40s) tt\n",title);
  return 1;
}


int CavPlotComment(char *mess) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"%%# mess %-40s\n",mess);
  fflush(l_meta);
  return 1;
}

int plotcm(char *mess) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"%%# %s\n",mess);
  return 1;
}

int CavPlotFlush(void) {
  if (l_meta == NULL) return 0;
  fflush(l_meta);
  return 1;
}

int CavPlotMeshDirection(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc /mesh {%d} def\n",num);
  return 1;
}

int CavPlotMeshBegin(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc save mesh %d ne { \n",num);
  fprintf(l_meta,
	  "cc -100 -100 m 0 0 rl 0 0 rl 0 0 rl 0 0 rl clip newpath } if\n");
  return 1;
}

int CavPlotMeshEnd(void) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc restore\n");
  return 1;
}

int CavPlotMatrix(void) {
  int i;
  if (l_meta == NULL) return 0;
  for(i=0;i<3;i++){
    fprintf(l_meta,"%%# RVMT%1d =%10.6f%10.6f%10.6f\n",
	    i+1,rvmt[i][kx],rvmt[i][ky],rvmt[i][kz]);
  }
  return 1;
}

int CavPlotColor(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cC %2d C\n",num);
  return 1;
}

==> cavity-5.5.1.tar.gz: cavity-5.5.1/src/plot.c <==
/* -*- C -*- */
/* $Id: plot.c 156 2007-03-07 07:34:00Z tnemoto $ */

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#include "cavity.h"

static int num_pen;
static double bx0=10000.0,by0=10000.0,bx1=-10000.0,by1=-10000.0;
static long boundingBoxPointer;

FILE *l_meta;

#define BoundingBoxString "%%%%BoundingBox: %6d %6d %6d %6d\n"

int CavPlotArc(double fx,double fy,double fz,double fr,
	    double fst,double fed,char *name1) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"ca  %8.3f %8.3f %8.3f %8.3f %8.3f a %% %8.3f    %-6s\n",
	  fx,fy,fr,degree(fst),degree(fed),fz,name1);
  if (bx1<fx+fr*cos(fst)) {
    bx1=fx+fr*cos(fst);
  }
  if (bx1<fx+fr*cos(fed)) {
    bx1=fx+fr*cos(fed);
  }
  if (fst<=PI && fed>=PI && bx0>fx-fr) {
    bx0=fx-fr;
  } 
  if (fx+fr*cos(fst)<bx0) {
    bx0=fx+fr*cos(fst);
  }
  if (fx+fr*cos(fed)<bx0) {
    bx0=fx+fr*cos(fed);
  }
  if (fst<=0.5*PI && fed>=0.5*PI && by1<fy+fr) {
    by1=fy+fr;
  } 
  if (by1<fy+fr*sin(fst)) {
    by1=fy+fr*sin(fst);
  }
  if (by1<fy+fr*sin(fed)) {
    by1=fy+fr*sin(fed);
  }
  if (fst<=1.5*PI && fed>=1.5*PI && by0>fy-fr) {
    by0=fy-fr;
  } 
  if (fy+fr*sin(fst)<by0) {
    by0=fy+fr*sin(fst);
  }
  if (fy+fr*sin(fed)<by0) {
    by0=fy+fr*sin(fed);
  }
  return 1;
}

int CavOpenPlotter(const char *fileName) {
#ifdef CREATION_TIME
  time_t tt;
  tt=time(NULL);
#endif  

  l_meta=fopen(fileName,"w+");
  if (l_meta==NULL) error("Can't Open Output File!");

  num_pen=-1;
  fprintf(l_meta,"%%!PS-Adobe-3.0 EPSF-3.0\n");
  fprintf(l_meta,"%%%%Creater: PS-META-FILE (Cavity %s)\n",VERSION);
#ifdef CREATION_TIME
  fprintf(l_meta,"%s %s\n","%%CreationDate: ",asctime(localtime(&tt)));
#endif
  /*
  c      fprintf(l_meta,"%s\n","%%DocumentMedia: ");
  c      fprintf(l_meta,"%s\n","%%DocumentPrinterRequired: ");
  c      fprintf(l_meta,"%s\n","%%For:");
  c      fprintf(l_meta,"%s\n","%%PageMedia:");
  c      fprintf(l_meta,"%s\n","%%PageRequirements:");
  */
  boundingBoxPointer=ftell(l_meta);
  fprintf(l_meta,BoundingBoxString,0.0,0.0,0.0,0.0);
  fprintf(l_meta,"%s\n","%%Pages: 1");
  fprintf(l_meta,"%s\n","%%EndComment");
  fprintf(l_meta,"%s\n","%%EndProlog");
  fprintf(l_meta,"save\n");
  fprintf(l_meta,"/color { false } def\n");
  fprintf(l_meta,"/color2 { false } def\n");
  fprintf(l_meta,"/penwidth {[0.001 0.001 0.005 0.050 0.001]} def\n");
  fprintf(l_meta,"/pengray {[0.000 0.000 0.000 0.000 0.700]} def\n");
  fprintf(l_meta,"/penred {[0.000 0.000 0.000 1.000 0.700]} def\n");
  fprintf(l_meta,"/pengreen {[0.000 0.000 0.000 0.000 0.700]} def\n");
  fprintf(l_meta,"/penblue {[0.000 1.000 0.000 0.000 0.700]} def\n");

  fprintf(l_meta,"/colorblue {[0 1 0 1 0 1 0 1]} def\n");
  fprintf(l_meta,"/colorred {[0 0 1 1 0 0 1 1]} def\n");
  fprintf(l_meta,"/colorgreen {[0 0 0 0 1 1 1 1]} def\n");

  fprintf(l_meta,"/m { moveto } def\n");
  fprintf(l_meta,"/s { stroke } def\n");
  fprintf(l_meta,"/l { /liney exch def /linex exch def\n");
  fprintf(l_meta,"    m linex liney lineto s} def\n");
  fprintf(l_meta,"/a { /arced exch def /arcst exch def\n");
  fprintf(l_meta,"/arcr exch def /arcy exch def\n");
  fprintf(l_meta,"/arcx exch def arcst cos arcr mul arcx add\n");
  fprintf(l_meta,"arcst sin arcr mul arcy add m \n");
  fprintf(l_meta,"arcx arcy arcr arcst arced arc s} def\n");
  fprintf(l_meta,"/C { /col exch def  color2 {\n");
  fprintf(l_meta,"colorred col get\n");
  fprintf(l_meta,"colorgreen col get\n");
  fprintf(l_meta,"colorblue col get\n");
  fprintf(l_meta,"setrgbcolor\n");
  fprintf(l_meta,"} if } def\n");
  fprintf(l_meta,"/p { dup penwidth exch get setlinewidth\n");
  fprintf(l_meta,"     color { /col exch def\n");
  fprintf(l_meta,"        penred col get pengreen col get\n");
  fprintf(l_meta,"        penblue col get setrgbcolor \n");
  fprintf(l_meta,"     } {pengray exch get setgray }\n");
  fprintf(l_meta,"     ifelse  } def\n");
  fprintf(l_meta,"/rm { rmoveto } def\n");
  fprintf(l_meta,"/rl { rlineto } def\n");
  fprintf(l_meta,"/tt {pop} def\n");
  fprintf(l_meta,"/ar { } def\n");
  fprintf(l_meta,"/cm {} def\n");
  fprintf(l_meta,"/cl {} def\n");
  fprintf(l_meta,"/crm {} def\n");
  fprintf(l_meta,"/crl {} def\n");
  fprintf(l_meta,"/cp {} def\n");
  fprintf(l_meta,"/ct {} def\n");
  fprintf(l_meta,"/ca {} def\n");
  fprintf(l_meta,"/cc {} def\n");
  fprintf(l_meta,"/cC {} def\n");
  fprintf(l_meta,"%% 50 50 m 500 0 rl 0 500 rl -500 0 rl\n");
  fprintf(l_meta,"%% closepath clip newpath\n");
  fprintf(l_meta,"%10.4f%10.4f%s\n",300.0,300.0," translate");
  fprintf(l_meta,"%10.4f%10.4f%s\n",50.0,50.0," scale");
  fprintf(l_meta,"%% View Control\n");
  fprintf(l_meta,"%10.4f%10.4f%s\n",scale,scale," scale");
  fprintf(l_meta,"%10.4f%10.4f%s\n",0.0,0.0," translate");
  fprintf(l_meta,"%s\n","%# StartData");
  fprintf(l_meta,"%%# ORIGIN=%10.6f%10.6f%10.6f\n",org[0],org[1],org[2]);
  fflush(l_meta);
  return 1;
}

int CavPlotLine(double fx,double fy,double fz,double fx2,double fy2,double fz2,
	    char *name1,char *name2){
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cl  %8.3f%8.3f%8.3f%8.3f l %% %8.3f%8.3f %-6s %-6s\n",
	  fx,fy,fx2,fy2,fz,fz2,name1,name2);
  if (fx<bx0) bx0=fx;
  if (fx2<bx0) bx0=fx2;
  if (fx>bx1) bx1=fx;
  if (fx2>bx1) bx1=fx2;
  if (fy<by0) by0=fy;
  if (fy2<by0) by0=fy2;
  if (fy>by1) by1=fy;
  if (fy2>by1) by1=fy2;
  return 1;
}

int CavPlotPen(int i){
  if (l_meta == NULL) return 0;
  if (i!=num_pen) {
    num_pen=i;
    fprintf(l_meta,"cp  %7d p\n",iPen[i-1]);
  }
  return 1;
}

int CavClosePlotter(void) {
  if (l_meta==NULL) return 0;
  fprintf(l_meta,"%%# EndData\n");
  fprintf(l_meta,"%%%%Trailer\n");
  fprintf(l_meta,"restore\n");
  fprintf(l_meta,"showpage\n");
  fprintf(l_meta,"%%%%EndTrailer\n");
  fprintf(l_meta,"%%%%EOF\n");
  fseek(l_meta,boundingBoxPointer,SEEK_SET);
  fprintf(l_meta,BoundingBoxString,
	  (int)(bx0*50*scale+300-1),
	  (int)(by0*50*scale+300-1),
	  (int)(bx1*50*scale+300+1),
	  (int)(by1*50*scale+300+1));
  fclose(l_meta);
  l_meta=NULL;
  return 1;
}

int CavPlotTitle(char *title) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"ct (%-40s) tt\n",title);
  return 1;
}


int CavPlotComment(char *mess) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"%%# mess %-40s\n",mess);
  fflush(l_meta);
  return 1;
}

int plotcm(char *mess) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"%%# %s\n",mess);
  return 1;
}

int CavPlotFlush(void) {
  if (l_meta == NULL) return 0;
  fflush(l_meta);
  return 1;
}

int CavPlotMeshDirection(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc /mesh {%d} def\n",num);
  return 1;
}

int CavPlotMeshBegin(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc save mesh %d ne { \n",num);
  fprintf(l_meta,
	  "cc -100 -100 m 0 0 rl 0 0 rl 0 0 rl 0 0 rl clip newpath } if\n");
  return 1;
}

int CavPlotMeshEnd(void) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cc restore\n");
  return 1;
}

int CavPlotMatrix(void) {
  int i;
  if (l_meta == NULL) return 0;
  for(i=0;i<3;i++){
    fprintf(l_meta,"%%# RVMT%1d =%10.6f%10.6f%10.6f\n",
	    i+1,rvmt[i][kx],rvmt[i][ky],rvmt[i][kz]);
  }
  return 1;
}

int CavPlotColor(int num) {
  if (l_meta == NULL) return 0;
  fprintf(l_meta,"cC %2d C\n",num);
  return 1;
}
